from parser_base import *
from mel_ast import *


class MELError(Exception):
    """Класс для исключений интерпретатора"""

    def __init__(self, message):
        super().__init__(message)


class MELParser(BaseParser):
    """Простейший интерпретатор MEL (Math Expr Lang).
    Вычисления производятся в процессе разбора

    Грамматика:
        num     -> <число>
        group   -> num | '(' expr ')'
        mult    -> group (('*' | '/') group)*
        add     -> mult (('+' | '-') mult)*
        expr    -> add
        start   -> expr
    """

    def __init__(self, text: str):
        """Конструктор
        :param text: текст программы
        """
        super().__init__(text)

    def num(self) -> NumNode:
        """num   -> <число>"""
        self.ws()
        pos = self.pos
        while self.curr.isdecimal() or self.curr == '.':
            self.pos += 1
        res = NumNode(float(self.text[pos:self.pos]))
        self.ws()
        return res

    def group(self) -> ExprNode:
        """ group   -> num | '(' expr ')' """
        if self.is_parse('('):
            self.parse('(')
            res = self.expr()
            self.parse(')')
            return res
        else:
            return self.num()

    def mult(self) -> BinOpNode:
        """ mult    -> group (('*' | '/') group)* """
        res = self.group()
        while self.is_parse('*', '/'):
            op = self.parse('*', '/')
            tmp = self.group()
            res = BinOpNode(BinOp(op), res, tmp)
        return res

    def add(self) -> BinOpNode:
        """ add     -> mult (('+' | '-') mult)* """
        res = self.mult()
        while self.is_parse('+', '-'):
            op = self.parse('+', '-')
            tmp = self.mult()
            res = BinOpNode(BinOp(op), res, tmp)
        return res

    def expr(self) -> ExprNode:
        """ expr    -> add """
        return self.add()

    def start(self) -> ExprNode:
        """ start   -> prog """
        return self.expr()

    @classmethod
    def parse_prog(cls, prog: str) -> ExprNode:
        parser = cls(str(prog))
        return parser.start()
